#ifndef _CLI_HELP_COMMAND_H_
#define _CLI_HELP_COMMAND_H_

#include <command/Command.h>
#include <environment/Environment.h>

namespace cli {

class HelpCommand : public Command {
	std::unique_ptr < cli::Arg > m_command;

public:
	HelpCommand ( std::unique_ptr < cli::Arg > command ) : m_command ( std::move ( command ) ) {
	}

	virtual Command::Result run ( Environment & environment ) const override {
		std::string command;
		if ( m_command != nullptr )
			command = m_command->eval ( environment );

		if ( command == "execute" ) {
			std::cout << "Execute command executes statements and either prints the result or writes the result to a file" << std::endl;
			std::cout << "statement_list:" << std::endl;
			std::cout << "statement ( | statement )* - at least one statement followed by a pipe separated sequence of other statements" << std::endl;
			std::cout << "" << std::endl;
			std::cout << "param:" << std::endl;
			std::cout << "-                   - a value from the previous result (PreviousResultParam)" << std::endl;
			std::cout << "<( statement_list ) - a statement list serving as a param (StatementParam)" << std::endl;
			std::cout << "( type ) param      - a casted parameter, type is an immediate value (CastParam)" << std::endl;
			std::cout << "( #type ) param     - a casted parameter, type identified by an environment variable (CastParam)" << std::endl;
			std::cout << "" << std::endl;
			std::cout << "identifier    - a string param - the value is a immediate string (ImmediateParam)" << std::endl;
			std::cout << "number        - a number param - the value is a immediate integer (ImmediateParam)" << std::endl;
			std::cout << "#identifier   - a string param - identified by an environment variable (ValueParam)" << std::endl;
			std::cout << "$identifier   - a value from a variable, identifier is an immediate value (VariableParam)" << std::endl;
			std::cout << "$#identifier  - a value from a variable, identified by an environment variable (VariableParam)" << std::endl;
			std::cout << "" << std::endl;
			std::cout << "<identifier   - a value from a xml file, identifier is an immediate value (FileParam)" << std::endl;
			std::cout << "<#identifier  - a value from a xml file, identified by an  environment variable (FileParam)" << std::endl;
			std::cout << "" << std::endl;
			std::cout << "statement:" << std::endl;
			std::cout << "name ( param )* output_spec  - a statement with params, name is an immediate value (SingleStatement)" << std::endl;
			std::cout << "( type ) statement           - the result of a statement is casted, type is an immediate value (CastedStatement)" << std::endl;
			std::cout << "( #type ) statement          - the result of a statement is casted, type indentified by an environment variable (CastedStatement)" << std::endl;
			std::cout << "" << std::endl;
			std::cout << "#identifier   - an immediate value from an environment (ValueStatement)" << std::endl;
			std::cout << "$identifier   - a value from a variable, identifier is an immediate value (VariableStatement)" << std::endl;
			std::cout << "$#identifier  - a value from a variable, identified by an environment variable (VariableStatement)" << std::endl;
			std::cout << "" << std::endl;
			std::cout << "<identifier   - a value from a xml file, identifier is immediate value (FileStatement)" << std::endl;
			std::cout << "<#identifier  - a value from a xml file, identified by an environment variable (FileStatement)" << std::endl;
			std::cout << "" << std::endl;
			std::cout << "output_spec:" << std::endl;
			std::cout << ">identifier   - a value to a xml file, identifier is an immediate value (ResultFileStatement)" << std::endl;
			std::cout << ">#identifier  - a value to a xml file, identified by an environment variable (ResultFileStatement)" << std::endl;
			std::cout << ">$identifier  - a value to a variable, identifier is an immediate value  (ResultVariableStatement)" << std::endl;
			std::cout << ">$#identifier - a value to a variable, identified by an environment variable (ResultVariableStatement)" << std::endl;
			std::cout << ">             - a value is discarded" << std::endl;
			std::cout << "              - an empty output specifier prints the result to the stdout (ResultPrintStatement)" << std::endl;
		} else if ( command == "" ) {
			std::cout << "Simple help for the query language" << std::endl;
			std::cout << "" << std::endl;
			std::cout << "command quit: quits the processor." << std::endl;
			std::cout << "command help: shows this help." << std::endl;
			std::cout << "command introspect: prints available algorithms, algorithm overloads, casts, and datatypes." << std::endl;
			std::cout << "command execute: executes statements" << std::endl;
			std::cout << "" << std::endl;
			std::cout << "for details use help of individual command" << std::endl;
		} else {
			std::cout << "The command " << command << " either does not exist or does not have a help entry." << std::endl;
		}
		return Command::Result::OK;
	}
};

} /* namespace cli */

#endif /* _CLI_HELP_COMMAND_H_ */
